/* linux/arch/arm/mach-s5pv210/mach-mango210.c
 *
 * Copyright (c) 2010 Samsung Electronics Co., Ltd.
 *		http://www.samsung.com/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/i2c.h>
#include <linux/i2c-gpio.h>
#include <linux/regulator/consumer.h>
#include <linux/regulator/fixed.h>
#include <linux/regulator/machine.h>
#include <linux/mfd/max8698.h>
#include <linux/rfkill-gpio.h>
#include <mach/power-domain.h>
#include <linux/init.h>
#include <linux/serial_core.h>
#include <linux/gpio_event.h>
#include <linux/sysdev.h>
#include <linux/smsc911x.h>
#include <linux/fb.h>
#include <linux/gpio.h>
#include <linux/videodev2.h>
#include <linux/clk.h>
#include <linux/delay.h>
#include <linux/pwm_backlight.h>
#include <linux/usb/ch9.h>
#include <linux/spi/spi.h>
#include <linux/gpio_keys.h>
#include <linux/leds.h>

#ifdef CONFIG_SENSORS_L3G4200D
#include <linux/l3g4200d.h>
#endif

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/setup.h>
#include <asm/mach-types.h>

#include <video/platform_lcd.h>

#include <mach/map.h>
#include <mach/regs-clock.h>
#include <mach/spi-clocks.h>
#include <mach/regs-fb.h>
#include <mach/gpio.h>

#ifdef CONFIG_VIDEO_S5K4BA
#include <media/s5k4ba_platform.h>
#undef	CAM_ITU_CH_A
#define	CAM_ITU_CH_B
#endif
#include <plat/regs-serial.h>
#include <plat/regs-srom.h>
#include <plat/gpio-cfg.h>
#include <plat/s3c64xx-spi.h>
#include <plat/s5pv210.h>
#include <plat/devs.h>
#include <plat/cpu.h>
#include <plat/adc.h>
#include <plat/ts.h>
#include <plat/ata.h>
#include <plat/iic.h>
#include <plat/keypad.h>
#include <plat/pm.h>
#include <plat/fb.h>
#include <plat/mfc.h>
#include <plat/s5p-time.h>
#include <plat/sdhci.h>
#include <plat/fimc.h>
#include <plat/csis.h>
#include <plat/jpeg.h>
#include <plat/clock.h> 
#include <plat/regs-otg.h>
#include <plat/otg.h>
#include <plat/ehci.h>
#include <plat/ohci.h>
#include <../../../drivers/video/samsung/s3cfb.h>
#include <mach/regs-gpio.h>
#include <mach/gpio.h>
#ifdef CONFIG_ANDROID_PMEM
#include <linux/android_pmem.h>
#endif
#include <plat/media.h>
#include <mach/media.h>
#include <mach/gpio-mango210.h>

#ifdef CONFIG_TOUCHSCREEN_EGALAX
#include <linux/i2c/egalax.h>
#define EETI_TS_DEV_NAME        "egalax_i2c"

static struct egalax_i2c_platform_data  egalax_platdata  = {
        .gpio_int = EGALAX_IRQ,
        .gpio_en = NULL,
        .gpio_rst = NULL,
};
#endif

/* Following are default values for UCON, ULCON and UFCON UART registers */
#define MANGO210_UCON_DEFAULT	(S3C2410_UCON_TXILEVEL |	\
				 S3C2410_UCON_RXILEVEL |	\
				 S3C2410_UCON_TXIRQMODE |	\
				 S3C2410_UCON_RXIRQMODE |	\
				 S3C2410_UCON_RXFIFO_TOI |	\
				 S3C2443_UCON_RXERR_IRQEN)

#define MANGO210_ULCON_DEFAULT	S3C2410_LCON_CS8

#define MANGO210_UFCON_DEFAULT	(S3C2410_UFCON_FIFOMODE |	\
				 S5PV210_UFCON_TXTRIG4 |	\
				 S5PV210_UFCON_RXTRIG4)

static struct s3c2410_uartcfg mango210_uartcfgs[] __initdata = {
	[0] = {
		.hwport		= 0,
		.flags		= 0,
		.ucon		= MANGO210_UCON_DEFAULT,
		.ulcon		= MANGO210_ULCON_DEFAULT,
		.ufcon		= MANGO210_UFCON_DEFAULT,
	},
	[1] = {
		.hwport		= 1,
		.flags		= 0,
		.ucon		= MANGO210_UCON_DEFAULT,
		.ulcon		= MANGO210_ULCON_DEFAULT,
		.ufcon		= MANGO210_UFCON_DEFAULT,
	},
	[2] = {
		.hwport		= 2,
		.flags		= 0,
		.ucon		= MANGO210_UCON_DEFAULT,
		.ulcon		= MANGO210_ULCON_DEFAULT,
		.ufcon		= MANGO210_UFCON_DEFAULT,
	},
	[3] = {
		.hwport		= 3,
		.flags		= 0,
		.ucon		= MANGO210_UCON_DEFAULT,
		.ulcon		= MANGO210_ULCON_DEFAULT,
		.ufcon		= MANGO210_UFCON_DEFAULT,
	},
};

#if defined(CONFIG_REGULATOR_MAX8698)
/* LDO */
static struct regulator_consumer_supply mango210_ldo3_consumer[] = {
	REGULATOR_SUPPLY("pd_io", "s3c-usbgadget"),
	REGULATOR_SUPPLY("pd_io", "s5p-ohci"),
	REGULATOR_SUPPLY("pd_io", "s5p-ehci"),
};

static struct regulator_consumer_supply mango210_ldo5_consumer[] = {
	REGULATOR_SUPPLY("AVDD", "0-001b"),
	REGULATOR_SUPPLY("DVDD", "0-001b"),
};

static struct regulator_consumer_supply mango210_ldo8_consumer[] = {
	REGULATOR_SUPPLY("pd_core", "s3c-usbgadget"),
	REGULATOR_SUPPLY("pd_core", "s5p-ohci"),
	REGULATOR_SUPPLY("pd_core", "s5p-ehci"),
};

static struct regulator_init_data mango210_ldo2_data = {
	.constraints	= {
		.name		= "VALIVE_1.1V",
		.min_uV		= 1100000,
		.max_uV		= 1100000,
		.apply_uV	= 1,
		.always_on	= 1,
		.state_mem	= {
			.enabled = 1,
		},
	},
};

static struct regulator_init_data mango210_ldo3_data = {
	.constraints	= {
		.name		= "VUOTG_D+VUHOST_D_1.1V",
		.min_uV		= 1100000,
		.max_uV		= 1100000,
		.apply_uV	= 1,
		.valid_ops_mask = REGULATOR_CHANGE_STATUS,
		.state_mem	= {
			.disabled = 1,
		},
	},
	.num_consumer_supplies	= ARRAY_SIZE(mango210_ldo3_consumer),
	.consumer_supplies	= mango210_ldo3_consumer,
};

static struct regulator_init_data mango210_ldo4_data = {
	.constraints	= {
		.name		= "V_MIPI_1.8V",
		.min_uV		= 1800000,
		.max_uV		= 1800000,
		.apply_uV	= 1,
		.always_on	= 1,
		.valid_ops_mask = REGULATOR_CHANGE_STATUS,
		.state_mem	= {
			.disabled = 1,
		},
	},
};

static struct regulator_init_data mango210_ldo5_data = {
	.constraints	= {
		.name		= "VMMC+VEXT_2.8V",
		.min_uV		= 2800000,
		.max_uV		= 2800000,
		.apply_uV	= 1,
		.always_on	= 1,
		.valid_ops_mask = REGULATOR_CHANGE_STATUS,
		.state_mem	= {
			.enabled = 1,
		},
	},
	.num_consumer_supplies	= ARRAY_SIZE(mango210_ldo5_consumer),
	.consumer_supplies	= mango210_ldo5_consumer,
};

static struct regulator_init_data mango210_ldo6_data = {
	.constraints	= {
		.name		= "VCC_2.6V",
		.min_uV		= 2600000,
		.max_uV		= 2600000,
		.apply_uV	= 1,
		.always_on	= 1,
		.valid_ops_mask	= REGULATOR_CHANGE_STATUS,
		.state_mem	 = {
			.disabled = 1,
		},
	},
};

static struct regulator_init_data mango210_ldo7_data = {
	.constraints	= {
		.name		= "VDAC_2.8V",
		.min_uV		= 2800000,
		.max_uV		= 2800000,
		.apply_uV	= 1,
		.valid_ops_mask	= REGULATOR_CHANGE_STATUS,
		.state_mem	= {
			.enabled = 1,
		},
	},
};

static struct regulator_init_data mango210_ldo8_data = {
	.constraints	= {
		.name		= "VUOTG_A+VUHOST_A_3.3V",
		.min_uV		= 3300000,
		.max_uV		= 3300000,
		.apply_uV	= 1,
		.valid_ops_mask	= REGULATOR_CHANGE_STATUS,
		.state_mem	= {
			.disabled = 1,
		},
	},
	.num_consumer_supplies	= ARRAY_SIZE(mango210_ldo8_consumer),
	.consumer_supplies	= mango210_ldo8_consumer,
};

static struct regulator_init_data mango210_ldo9_data = {
	.constraints	= {
		.name		= "VADC+VSYS+VKEY_2.8V",
		.min_uV		= 2800000,
		.max_uV		= 2800000,
		.apply_uV	= 1,
		.always_on	= 1,
		.state_mem	= {
			.enabled = 1,
		},
	},
};

/* BUCK */
static struct regulator_consumer_supply mango210_buck1_consumer =
	REGULATOR_SUPPLY("vddarm", NULL);

static struct regulator_consumer_supply mango210_buck2_consumer =
	REGULATOR_SUPPLY("vddint", NULL);

static struct regulator_init_data mango210_buck1_data = {
	.constraints	= {
		.name		= "VCC_ARM",
		.min_uV		= 750000,
		.max_uV		= 1500000,
		.apply_uV	= 1,
		.valid_ops_mask	= REGULATOR_CHANGE_VOLTAGE |
				  REGULATOR_CHANGE_STATUS,
		.state_mem	= {
			.uV	= 1250000,
			.mode	= REGULATOR_MODE_NORMAL,
			.disabled = 1,
		},
	},
	.num_consumer_supplies	= 1,
	.consumer_supplies	= &mango210_buck1_consumer,
};

static struct regulator_init_data mango210_buck2_data = {
	.constraints	= {
		.name		= "VCC_INTERNAL",
		.min_uV		= 950000,
		.max_uV		= 1200000,
		.valid_ops_mask	= REGULATOR_CHANGE_VOLTAGE |
				  REGULATOR_CHANGE_STATUS,
		.state_mem	= {
			.uV	= 1100000,
			.mode	= REGULATOR_MODE_NORMAL,
			.disabled = 1,
		},
	},
	.num_consumer_supplies	= 1,
	.consumer_supplies	= &mango210_buck2_consumer,
};

static struct regulator_init_data mango210_buck3_data = {
	.constraints	= {
		.name		= "VCC_MEM",
		.min_uV		= 1800000,
		.max_uV		= 1800000,
		.always_on	= 1,
		.apply_uV	= 1,
		.state_mem	= {
			.uV	= 1800000,
			.mode	= REGULATOR_MODE_NORMAL,
			.enabled = 1,
		},
	},
};

static struct max8698_regulator_data mango210_regulators[] = {
	{ MAX8698_LDO2,  &mango210_ldo2_data },
	{ MAX8698_LDO3,  &mango210_ldo3_data },
	{ MAX8698_LDO4,  &mango210_ldo4_data },
	{ MAX8698_LDO5,  &mango210_ldo5_data },
	{ MAX8698_LDO6,  &mango210_ldo6_data },
	{ MAX8698_LDO7,  &mango210_ldo7_data },
	{ MAX8698_LDO8,  &mango210_ldo8_data },
	{ MAX8698_LDO9,  &mango210_ldo9_data },
	{ MAX8698_BUCK1, &mango210_buck1_data },
	{ MAX8698_BUCK2, &mango210_buck2_data },
	{ MAX8698_BUCK3, &mango210_buck3_data },
};

static struct max8698_platform_data mango210_max8698_pdata = {
	.num_regulators = ARRAY_SIZE(mango210_regulators),
	.regulators     = mango210_regulators,

	/* 1GHz default voltage */
	.dvsarm1        = 0xa,  /* 1.25v */
	.dvsarm2        = 0x9,  /* 1.20V */
	.dvsarm3        = 0x6,  /* 1.05V */
	.dvsarm4        = 0x4,  /* 0.95V */
	.dvsint1        = 0x7,  /* 1.10v */
	.dvsint2        = 0x5,  /* 1.00V */
	.set1       = S5PV210_GPH1(6),
	.set2       = S5PV210_GPH1(7),
	.set3       = S5PV210_GPH0(4),
};
#endif
//CRZ fix USB host
#ifdef CONFIG_USB_SUPPORT
static void __init mango210_usb_host_set(void)
{
	u32 reg;

        reg= __raw_readl(S5PV210_ETC2_BASE+0x8);
        //printk("CRZ ETC2_----------------------------BASE+8 reg=0x%x \n",reg);
        reg |=0x4000;
        __raw_writel(reg, (S5PV210_ETC2_BASE+0x8));
        reg= __raw_readl(S5PV210_ETC2_BASE+0x8);
        //printk("CRZ ETC2_After----------------------------BASE+8 reg=0x%x \n",reg);
}
#endif

#ifdef CONFIG_SMSC911X
static struct smsc911x_platform_config smsc911x_config = {
      .irq_polarity  = SMSC911X_IRQ_POLARITY_ACTIVE_LOW,
      .irq_type    = SMSC911X_IRQ_TYPE_PUSH_PULL,
      .flags  = SMSC911X_USE_16BIT || SMSC911X_FORCE_INTERNAL_PHY,
      .phy_interface  = PHY_INTERFACE_MODE_MII,
      .mac            = {0x00, 0x80, 0x00, 0x23, 0x45, 0x67},
};

static struct resource s5p_smsc911x_resources[] = {
      [0] = {
              .start  = S5PV210_PA_SROM_BANK1,
              .end    = S5PV210_PA_SROM_BANK1 + SZ_1M - 1,
              .flags  = IORESOURCE_MEM,
      },
      [1] = {
              .start = IRQ_EINT(11),
              .end   = IRQ_EINT(11),
              .flags = IORESOURCE_IRQ | IRQF_TRIGGER_LOW,
        },
};

struct platform_device s5p_device_smsc911x = {
	.name       = "smsc911x",
	.id     =  -1,
	.num_resources  = ARRAY_SIZE(s5p_smsc911x_resources),
	.resource   = s5p_smsc911x_resources,
	.dev        = {
		.platform_data = &smsc911x_config,
	}
};
#endif

#ifdef CONFIG_REGULATOR
static struct regulator_consumer_supply mango210_b_pwr_5v_consumers[] = {
        {
                /* WM8580 */
                .supply         = "PVDD",
                .dev_name       = "0-001b",
        },
};

static struct regulator_init_data mango210_b_pwr_5v_data = {
        .constraints = {
                .always_on = 1,
        },
        .num_consumer_supplies  = ARRAY_SIZE(mango210_b_pwr_5v_consumers),
        .consumer_supplies      = mango210_b_pwr_5v_consumers,
};

static struct fixed_voltage_config mango210_b_pwr_5v_pdata = {
        .supply_name    = "B_PWR_5V",
        .microvolts     = 5000000,
        .init_data      = &mango210_b_pwr_5v_data,
	.gpio		= -1,
};

static struct platform_device mango210_b_pwr_5v = {
        .name          = "reg-fixed-voltage",
        .id            = -1,
        .dev = {
                .platform_data = &mango210_b_pwr_5v_pdata,
        },
};
#endif
#ifdef CONFIG_TOUCHSCREEN_EGALAX
static struct i2c_gpio_platform_data i2c5_platdata = {
        .sda_pin                = S5PV210_GPB(6),
        .scl_pin                = S5PV210_GPB(7),
        .udelay                 = 2,
        .sda_is_open_drain      = 0,
        .scl_is_open_drain      = 0,
        .scl_is_output_only     = 0.
};

//static struct platform_device   s3c_device_i2c5 = {
struct platform_device   s3c_device_i2c5 = {
        .name                   = "i2c-gpio",
        .id                     = 5,
        .dev.platform_data      = &i2c5_platdata,
};

static struct i2c_board_info i2c_devs5[] __initdata = {
        {
                I2C_BOARD_INFO(EETI_TS_DEV_NAME, 0x04),
                .platform_data = &egalax_platdata,
                .irq = IRQ_EINT6,
        },
};
#endif

#ifdef CONFIG_BATTERY_S3C
struct platform_device sec_device_battery = {
        .name   = "sec-fake-battery",
        .id = -1,
};
#endif

#ifdef  CONFIG_FB_MANGO_7INCH_AT070TNA2
#define S5PV210_LCD_WIDTH 1024 
#define S5PV210_LCD_HEIGHT 600 
#define NUM_BUFFER 4
#endif
#ifdef CONFIG_FB_MANGO_101_LTN101AL03
#define S5PV210_LCD_WIDTH 1280
#define S5PV210_LCD_HEIGHT 800
#define NUM_BUFFER 4
#endif

#ifdef CONFIG_FB_S3C_LB070WV6
#define S5PV210_LCD_WIDTH 800
#define S5PV210_LCD_HEIGHT 480
#define NUM_BUFFER 4
#endif

#ifdef CONFIG_FB_S3C_LSA40AT9001
#define S5PV210_LCD_WIDTH 800
#define S5PV210_LCD_HEIGHT 600
#define NUM_BUFFER 4
#endif

#ifdef CONFIG_FB_S3C_LTE480WV
#define S5PV210_LCD_WIDTH 800
#define S5PV210_LCD_HEIGHT 480
#define NUM_BUFFER 4
#endif

#ifdef CONFIG_FB_S3C_101WA01S
//#define S5PV210_LCD_WIDTH 1366
#define S5PV210_LCD_WIDTH 1024
#define S5PV210_LCD_HEIGHT 768
#endif

#ifdef CONFIG_FB_S3C_TL2796
#define S5PV210_LCD_WIDTH 1024 //1280
#define S5PV210_LCD_HEIGHT 768 //800
#define NUM_BUFFER 8
#endif
#define  S5PV210_VIDEO_SAMSUNG_MEMSIZE_FIMC0 (6144 * SZ_1K)
#define  S5PV210_VIDEO_SAMSUNG_MEMSIZE_FIMC1 (9900 * SZ_1K)
#define  S5PV210_VIDEO_SAMSUNG_MEMSIZE_FIMC2 (6144 * SZ_1K)
#define  S5PV210_VIDEO_SAMSUNG_MEMSIZE_MFC0 (36864 * SZ_1K)
#define  S5PV210_VIDEO_SAMSUNG_MEMSIZE_MFC1 (36864 * SZ_1K)
#if 1//CRZ hdmi 
#define  S5PV210_VIDEO_SAMSUNG_MEMSIZE_FIMD (1920 * \
                                             1080 * NUM_BUFFER * \
                                             (CONFIG_FB_S3C_NR_BUFFERS + \
                                                 (CONFIG_FB_S3C_NUM_OVLY_WIN * \
                                                  CONFIG_FB_S3C_NUM_BUF_OVLY_WIN)))
#else
#define  S5PV210_VIDEO_SAMSUNG_MEMSIZE_FIMD (S5PV210_LCD_WIDTH * \
                                             S5PV210_LCD_HEIGHT * NUM_BUFFER * \
                                             (CONFIG_FB_S3C_NR_BUFFERS + \
                                                 (CONFIG_FB_S3C_NUM_OVLY_WIN * \
                                                  CONFIG_FB_S3C_NUM_BUF_OVLY_WIN)))
#endif

#define  S5PV210_VIDEO_SAMSUNG_MEMSIZE_JPEG (8192 * SZ_1K)
#define  S5PV210_ANDROID_PMEM_MEMSIZE_PMEM_GPU1 (1800 * SZ_1K)
#define  S5PV210_VIDEO_SAMSUNG_MEMSIZE_G2D (8192 * SZ_1K)

static struct s5p_media_device s5pv210_media_devs[] = {
        [0] = {
                .id = S5P_MDEV_MFC,
                .name = "mfc",
                .bank = 0,
                .memsize = S5PV210_VIDEO_SAMSUNG_MEMSIZE_MFC0,
                .paddr = 0,
        },
        [1] = {
                .id = S5P_MDEV_MFC,
                .name = "mfc",
                .bank = 1,
                .memsize = S5PV210_VIDEO_SAMSUNG_MEMSIZE_MFC1,
                .paddr = 0,
        },
        [2] = {
                .id = S5P_MDEV_FIMC0,
                .name = "fimc0",
                .bank = 1,
                .memsize = S5PV210_VIDEO_SAMSUNG_MEMSIZE_FIMC0,
                .paddr = 0,
        },
        [3] = {
                .id = S5P_MDEV_FIMC1,
                .name = "fimc1",
                .bank = 1,
                .memsize = S5PV210_VIDEO_SAMSUNG_MEMSIZE_FIMC1,
                .paddr = 0,
        },
        [4] = {
                .id = S5P_MDEV_FIMC2,
                .name = "fimc2",
                .bank = 1,
                .memsize = S5PV210_VIDEO_SAMSUNG_MEMSIZE_FIMC2,
                .paddr = 0,
        },
        [5] = {
                .id = S5P_MDEV_JPEG,
                .name = "jpeg",
                .bank = 0,
                .memsize = S5PV210_VIDEO_SAMSUNG_MEMSIZE_JPEG,
                .paddr = 0,
        },
	[6] = {
                .id = S5P_MDEV_FIMD,
                .name = "fimd",
                .bank = 1,
                .memsize = S5PV210_VIDEO_SAMSUNG_MEMSIZE_FIMD,
                .paddr = 0,
        },
	[7] = {
                .id = S5P_MDEV_PMEM_GPU1,
                .name = "pmem_gpu1",
                .bank = 0, /* OneDRAM */
                .memsize = S5PV210_ANDROID_PMEM_MEMSIZE_PMEM_GPU1,
                .paddr = 0,
        },
	[8] = {
		.id = S5P_MDEV_G2D,
		.name = "g2d",
		.bank = 0,
		.memsize = S5PV210_VIDEO_SAMSUNG_MEMSIZE_G2D,
		.paddr = 0,
	},
#if 0 
        [9] = {
                .id = S5P_MDEV_PMEM,
                .name = "pmem",
                .bank = 0, /* OneDRAM */
                .memsize = (CONFIG_ANDROID_PMEM_MEMSIZE_PMEM * SZ_1K),
                .paddr = 0,
        },
        [10] = {
                .id = S5P_MDEV_PMEM_ADSP,
                .name = "pmem_adsp",
                .bank = 0, /* OneDRAM */
                .memsize = (CONFIG_ANDROID_PMEM_MEMSIZE_PMEM_ADSP * SZ_1K),
                .paddr = 0,
        },
#endif
};

#ifdef CONFIG_ANDROID_PMEM
static struct android_pmem_platform_data pmem_pdata = {
        .name = "pmem",
        .no_allocator = 1,
        .cached = 1,
        .start = 0,
        .size = 0,
};

static struct android_pmem_platform_data pmem_gpu1_pdata = {
        .name = "pmem_gpu1",
        .no_allocator = 1,
        .cached = 1,
        .buffered = 1,
        .start = 0,
        .size = 0,
};
 
static struct android_pmem_platform_data pmem_adsp_pdata = {
        .name = "pmem_adsp",
        .no_allocator = 1,
        .cached = 1,
        .buffered = 1,
        .start = 0,
        .size = 0,
};      

static struct platform_device pmem_device = {
        .name = "android_pmem",
        .id = 0,
        .dev = { .platform_data = &pmem_pdata },
};

static struct platform_device pmem_gpu1_device = {
        .name = "android_pmem",
        .id = 1,
        .dev = { .platform_data = &pmem_gpu1_pdata },
};

static struct platform_device pmem_adsp_device = {
        .name = "android_pmem",
        .id = 2,
        .dev = { .platform_data = &pmem_adsp_pdata },
};

static void __init android_pmem_set_platdata(void)
{
        pmem_pdata.start = (u32)s5p_get_media_memory_bank(S5P_MDEV_PMEM, 0);
        pmem_pdata.size = (u32)s5p_get_media_memsize_bank(S5P_MDEV_PMEM, 0);

        pmem_gpu1_pdata.start =
                (u32)s5p_get_media_memory_bank(S5P_MDEV_PMEM_GPU1, 0);
        pmem_gpu1_pdata.size =
                (u32)s5p_get_media_memsize_bank(S5P_MDEV_PMEM_GPU1, 0);

        pmem_adsp_pdata.start =
                (u32)s5p_get_media_memory_bank(S5P_MDEV_PMEM_ADSP, 0);
        pmem_adsp_pdata.size =
                (u32)s5p_get_media_memsize_bank(S5P_MDEV_PMEM_ADSP, 0);
}
#endif


static void mango210_fb_lcd_set_power(struct plat_lcd_data *pd,
					unsigned int power)
{
	if (power) {
#if !defined(CONFIG_BACKLIGHT_PWM)
		gpio_request(S5PV210_GPD0(0), "GPD0_0");
		gpio_direction_output(S5PV210_GPD0(0), 1);
		gpio_free(S5PV210_GPD0(0));
#endif

		/* fire POWER Enable */
		gpio_request(S5PV210_GPC1(4), "GPC1_4");

		gpio_direction_output(S5PV210_GPC1(4), 1);

		gpio_set_value(S5PV210_GPC1(4), 1);

		gpio_free(S5PV210_GPC1(4));
	} else {
#if !defined(CONFIG_BACKLIGHT_PWM)
		gpio_request(S5PV210_GPD0(0), "GPD0_0");
		gpio_direction_output(S5PV210_GPD0(0), 0);
		gpio_free(S5PV210_GPD0(0));
#endif
	}
}

static struct plat_lcd_data mango210_lcd_fb_lcd_data = {
	.set_power	= mango210_fb_lcd_set_power,
};

static struct platform_device mango210_lcd_fb_lcd = {
	.name			= "platform-lcd",
	.dev.parent		= &s3c_device_fb.dev,
	.dev.platform_data	= &mango210_lcd_fb_lcd_data,
};

#if 0
static struct s3c_fb_pd_win mango210_fb_win0 = {
	.win_mode = {
		.left_margin	= 13,
		.right_margin	= 8,
		.upper_margin	= 7,
		.lower_margin	= 5,
		.hsync_len	= 3,
		.vsync_len	= 1,
		.xres		= 800,
		.yres		= 480,
	},
	.max_bpp	= 32,
	.default_bpp	= 24,
};

static struct s3c_fb_platdata mango210_lcd0_pdata __initdata = {
	.win[0]		= &mango210_fb_win0,
	.vidcon0	= VIDCON0_VIDOUT_RGB | VIDCON0_PNRMODE_RGB,
	.vidcon1	= VIDCON1_INV_HSYNC | VIDCON1_INV_VSYNC,
	.setup_gpio	= s5pv210_fb_gpio_setup_24bpp,
};
#endif 
//#define S5PV210_LCD_WIDTH  800
//#define S5PV210_LCD_HEIGHT 480
#ifdef CONFIG_FB_S3C_LTE480WV
static struct s3cfb_lcd lte480wv = {
        .width = S5PV210_LCD_WIDTH,
        .height = S5PV210_LCD_HEIGHT,
        .bpp = 32,
        .freq = 60,

        .timing = {
                .h_fp = 8,
                .h_bp = 13,
                .h_sw = 3,
                .v_fp = 5,
                .v_fpe = 1,
                .v_bp = 7,
                .v_bpe = 1,
                .v_sw = 1,
        },
        .polarity = {
                .rise_vclk = 0,
                .inv_hsync = 1,
                .inv_vsync = 1,
                .inv_vden = 0,
        },
};
#endif
#ifdef CONFIG_FB_MANGO_7INCH_AT070TNA2  
static struct s3cfb_lcd mango_7inch_at070tna2 = {
        .width = S5PV210_LCD_WIDTH,
        .height = S5PV210_LCD_HEIGHT,
        .bpp = 32,
        .freq = 65,

        .timing = {
                .h_fp   = 60,
                .h_bp   = 60,
                .h_sw   = 200,
                .v_fp   = 5,
                .v_fpe  = 1,
                .v_bp   = 5,
                .v_bpe  = 1,
                .v_sw   = 25,
        },
        .polarity = {
                .rise_vclk = 1,
                .inv_hsync = 1,
                .inv_vsync = 1,
                .inv_vden = 0,
        },
};
#endif 

#ifdef CONFIG_FB_MANGO_101_LTN101AL03
static struct s3cfb_lcd  mango_10_1inch_ltn101al03= {
        .width  = S5PV210_LCD_WIDTH,//1280,
        .height = S5PV210_LCD_HEIGHT,//800,
        .bpp    = 32,
        .freq   = 65,

        .timing = {
                .h_fp   = 32,//64,
                .h_bp   = 32, //64,
                .h_sw   = 64,//128,
                .v_fp   = 8,
                .v_fpe  = 0,//1,
                .v_bp   = 4,
                .v_bpe  = 0,
                .v_sw   = 4,
        },

        .polarity = {
                .rise_vclk      = 1,
                .inv_hsync      = 1,
                .inv_vsync      = 1,
                .inv_vden       = 0,
        },
};
#endif 
#ifdef CONFIG_FB_S3C_LB070WV6
static struct s3cfb_lcd lb070wv6 = {
        .width = S5PV210_LCD_WIDTH,
        .height = S5PV210_LCD_HEIGHT,
        .bpp = 32,
        .freq = 60,

        .timing = {
                .h_fp   = 64,
                .h_bp   = 64,
                .h_sw   = 128,
                .v_fp   = 16,
                .v_fpe  = 1,
                .v_bp   = 14,
                .v_bpe  = 1,
                .v_sw   = 1,
        },
        .polarity = {
                .rise_vclk = 0,
                .inv_hsync = 1,
                .inv_vsync = 1,
                .inv_vden = 0,
        },
};
#endif
#ifdef CONFIG_FB_S3C_LSA40AT9001
static struct s3cfb_lcd lsa40at9001 = {
        .width = S5PV210_LCD_WIDTH,
        .height = S5PV210_LCD_HEIGHT,
        .bpp = 24,
        .freq = 65,
     .timing = {
                .h_fp   = 210,
                .h_bp   = 46,
                .h_sw   = 28,
                .v_fp   = 12,
                .v_fpe  = 1,
                .v_bp   = 23,
                .v_bpe  = 1,
                .v_sw   = 10,
        },
 
        .polarity = {
                .rise_vclk      = 0,
                .inv_hsync      = 1,
                .inv_vsync      = 1,
                .inv_vden       = 0,
        },

};
#endif
#ifdef CONFIG_FB_S3C_TL2796
static struct s3cfb_lcd lb070wv6 = {
	.width = S5PV210_LCD_WIDTH,
	.height = S5PV210_LCD_HEIGHT,
	.bpp = 32,
	.freq = 60,

	.timing = {
		.h_fp = 44,
		.h_bp = 50,
		.h_sw = 16,
		.v_fp = 10,
		.v_fpe = 0,
		.v_bp = 5,
		.v_bpe = 0,
		.v_sw = 3,
	},
	.polarity = {
		.rise_vclk = 1,
		.inv_hsync = 0,
		.inv_vsync = 1,
		.inv_vden = 0,
	},
};
#endif

static void fb_lcd_cfg_gpio(struct platform_device *pdev)
{
        int i;

        for (i = 0; i < 8; i++) {
                s3c_gpio_cfgpin(S5PV210_GPF0(i), S3C_GPIO_SFN(2));
                s3c_gpio_setpull(S5PV210_GPF0(i), S3C_GPIO_PULL_NONE);
        }

        for (i = 0; i < 8; i++) {
                s3c_gpio_cfgpin(S5PV210_GPF1(i), S3C_GPIO_SFN(2));
                s3c_gpio_setpull(S5PV210_GPF1(i), S3C_GPIO_PULL_NONE);
        }

        for (i = 0; i < 8; i++) {
                s3c_gpio_cfgpin(S5PV210_GPF2(i), S3C_GPIO_SFN(2));
                s3c_gpio_setpull(S5PV210_GPF2(i), S3C_GPIO_PULL_NONE);
        }
        for (i = 0; i < 4; i++) {
                s3c_gpio_cfgpin(S5PV210_GPF3(i), S3C_GPIO_SFN(2));
                s3c_gpio_setpull(S5PV210_GPF3(i), S3C_GPIO_PULL_NONE);
        }

        /* mDNIe SEL: why we shall write 0x2 ? */
        writel(0x2, S5P_MDNIE_SEL);

#ifndef CONFIG_FB_S3C_TL2796
        /* drive strength to max */
        writel(0xffffffff, S5PV210_GPF0_BASE + 0xc);
        writel(0xffffffff, S5PV210_GPF1_BASE + 0xc);
        writel(0xffffffff, S5PV210_GPF2_BASE + 0xc);
        writel(0x000000ff, S5PV210_GPF3_BASE + 0xc);
#else
        writel(0xC0, S5PV210_GPF0_BASE + 0xc);
#endif
}



#define S5PV210_GPD_0_0_TOUT_0  (0x2)
#define S5PV210_GPD_0_1_TOUT_1  (0x2 << 4)
#define S5PV210_GPD_0_2_TOUT_2  (0x2 << 8)
#define S5PV210_GPD_0_3_TOUT_3  (0x2 << 12)
static int fb_lcd_backlight_on(struct platform_device *pdev)
{
        int err;

        err = gpio_request(S5PV210_GPD0(0), "GPD0_0");

        if (err) {
                printk(KERN_ERR "failed to request GPD0 for "
                        "lcd backlight control\n");
                return err;
        }

        gpio_direction_output(S5PV210_GPD0(0), 1);

        s3c_gpio_cfgpin(S5PV210_GPD0(0), S5PV210_GPD_0_1_TOUT_1);

        gpio_free(S5PV210_GPD0(0));

        return 0;
}


static int fb_lcd_backlight_off(struct platform_device *pdev, int onoff)
{
        int err;

        err = gpio_request(S5PV210_GPD0(0), "GPD0_0");

        if (err) {
                printk(KERN_ERR "failed to request GPD0 for "
                                "lcd backlight control\n");
                return err;
        }

        gpio_direction_output(S5PV210_GPD0(0), 0);
        gpio_free(S5PV210_GPD0(0));

        return 0;
}

#ifdef CONFIG_FB_S3C_TL2796
void lcd_cfg_gpio_early_suspend(void)
{
	return;
}

void lcd_cfg_gpio_late_resume(void)
{
	    return;
}
#endif

static int fb_lcd_reset_lcd(struct platform_device *pdev)
{
        return 0;
}

static struct s3c_platform_fb fb_lcd_fb_data __initdata = {
        .hw_ver = 0x62,
        .clk_name       = "sclk_fimd",
        .nr_wins = 5,
        .default_win = CONFIG_FB_S3C_DEFAULT_WINDOW,
        .swap = FB_SWAP_WORD | FB_SWAP_HWORD,
#ifdef CONFIG_FB_S3C_LSA40AT9001
        .lcd = &lsa40at9001,
#endif

#ifdef CONFIG_FB_S3C_LB070WV6
        .lcd = &lb070wv6,
#endif

#ifdef CONFIG_FB_MANGO_7INCH_AT070TNA2
        .lcd = &mango_7inch_at070tna2,
#endif

#ifdef CONFIG_FB_MANGO_101_LTN101AL03
	.lcd = &mango_10_1inch_ltn101al03,
#endif

        .cfg_gpio       = fb_lcd_cfg_gpio,
        .backlight_on   = fb_lcd_backlight_on,
        .backlight_onoff    = fb_lcd_backlight_off,
        .reset_lcd      = fb_lcd_reset_lcd,
};

static int mango210_backlight_init(struct device *dev)
{
	//need to check the calling function for this function and remove the call.
	return 0;
}

static void mango210_backlight_exit(struct device *dev)
{
	s3c_gpio_cfgpin(S5PV210_GPD0(0), S3C_GPIO_OUTPUT);
	gpio_free(S5PV210_GPD0(0));
}

static struct platform_pwm_backlight_data mango210_backlight_data = {
	.pwm_id		= 3,
	.max_brightness	= 255,
	.dft_brightness	= 255,
	.lth_brightness = 50,
	.pwm_period_ns	= 25000,
	.init		= mango210_backlight_init,
	.exit		= mango210_backlight_exit,
};

static struct platform_device mango210_backlight_device = {
	.name		= "pwm-backlight",
	.dev		= {
		.parent		= &s3c_device_timer[3].dev,
		.platform_data	= &mango210_backlight_data,
	},
};

struct s3c_adc_mach_info {
        /* if you need to use some platform data, add in here*/
        int delay;
        int presc;
        int resolution;
};

/*
 * mangov210 button
 */
static struct gpio_keys_button mango210_buttons[] = {
        {
                .gpio                   = S5PV210_GPH0(1),
                .code                   = 48, // BACK,
                .desc                   = "Back",
                .active_low             = 1,
                .debounce_interval      = 5,
                .type                   = EV_KEY,
        },
        {
                .gpio                   = S5PV210_GPH0(2),
                .code                   = 30, // HOME,
                .desc                   = "Menu",
                .active_low             = 1,
                .debounce_interval      = 5,
                .type                   = EV_KEY,
        },
};

static struct gpio_keys_platform_data mango210_buttons_data  = {
        .buttons        = mango210_buttons,
        .nbuttons       = ARRAY_SIZE(mango210_buttons),
};

static struct platform_device mango210_buttons_device  = {
        .name           = "gpio-keys",
        .id             = 0,
        .num_resources  = 0,
        .dev            = {
                .platform_data  = &mango210_buttons_data,
        }
};

static struct gpio_led mango210_leds[] = {
        { /* */
                .name                   = "led0",
                .gpio                   = S5PV210_GPJ3(7),
                .active_low             = 1,
                .default_trigger        = "default-on",
        },
        { /* */
                .name                   = "led1",
                .gpio                   = S5PV210_GPJ4(0),
                .active_low             = 1,
                .default_trigger        = "heartbeat",
        },
};

static struct gpio_led_platform_data mango210_led_data = {
        .num_leds = ARRAY_SIZE(mango210_leds),
        .leds = mango210_leds,
};

static struct platform_device mango210_leds_device = {
        .name                   = "leds-gpio",
        .id                     = -1,
        .dev.platform_data      = &mango210_led_data,
};

#define S3C_GPIO_SETPIN_ZERO         0
#define S3C_GPIO_SETPIN_ONE          1
#define S3C_GPIO_SETPIN_NONE         2

struct gpio_init_data {
        uint num;
        uint cfg;
        uint val;
        uint pud;
        uint drv;
};

static struct gpio_init_data mango210_init_gpios[] = {
        { /* BT_EN */
                .num    = S5PV210_GPC1(0),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* WIFI_EN */
                .num    = S5PV210_GPC1(1),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* BT_WAKEUPn */
                .num    = S5PV210_GPC1(2),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* WIFI_WAKEUPn */
                .num    = S5PV210_GPC1(3),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* LCD_PWR_EN */
                .num    = S5PV210_GPC1(4),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* LCD_BL_DIM */
                .num    = S5PV210_GPD0(0),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
#if 1
        }, { /* KEY_DIN0 */
                .num    = S5PV210_GPH0(1),
                .cfg    = S3C_GPIO_INPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* KEY_DIN1 */
                .num    = S5PV210_GPH0(2),
                .cfg    = S3C_GPIO_INPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
#endif
        }, { /* EINT(3) */
                .num    = S5PV210_GPH0(3),
                .cfg    = S3C_GPIO_INPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* EINT(4) Gyroscope RDRY */
                .num    = S5PV210_GPH0(4),
                .cfg    = S3C_GPIO_INPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* LAN_IRQ */
                .num    = S5PV210_GPH1(3),
                .cfg    = 0xFF,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* IND_LED1 */
                .num    = S5PV210_GPJ3(7),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* IND_LED2 */
                .num    = S5PV210_GPJ4(0),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* CAM_A_PWRDN */
                .num    = S5PV210_GPJ4(1),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* CAM_A_RESETn */
                .num    = S5PV210_GPJ4(2),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* CAM_B_PWRDN */
                .num    = S5PV210_GPJ4(3),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        }, { /* CAM_B_RESETn */
                .num    = S5PV210_GPJ4(4),
                .cfg    = S3C_GPIO_OUTPUT,
                .val    = S3C_GPIO_SETPIN_NONE,
                .pud    = S3C_GPIO_PULL_UP,
                .drv    = S3C_GPIO_DRVSTR_1X,
        },
};

static void mango210_board_cfg_gpio(void)
{
        u32 i, gpio;

        for (i = 0; i < ARRAY_SIZE(mango210_init_gpios); i++) {
                gpio = mango210_init_gpios[i].num;
                if (gpio <= S5PV210_MP05(7)) {
                        s3c_gpio_cfgpin(gpio, mango210_init_gpios[i].cfg);
                        s3c_gpio_setpull(gpio, mango210_init_gpios[i].pud);

                        if (mango210_init_gpios[i].val != S3C_GPIO_SETPIN_NONE)
                                gpio_set_value(gpio, mango210_init_gpios[i].val);

                        s3c_gpio_set_drvstrength(gpio, mango210_init_gpios[i].drv);
                }
        }

}


static void __init mango210_sound_init(void)
{
        u32 reg;
#if 0
        reg = __raw_readl(S5P_CLK_OUT);
        reg &= ~(0x1f << 12);
        reg &= ~(0xf << 20);
        reg |= 0x12 << 12;
        reg |= 0x1  << 20;
        __raw_writel(reg, S5P_CLK_OUT);

        reg = __raw_readl(S5P_OTHERS);
        reg &= ~(0x3 << 8);
        reg |= 0x0 << 8;
        __raw_writel(reg, S5P_OTHERS);
#endif
#if 1
        reg = __raw_readl(S5P_OTHERS);
        reg &= ~(0x3 << 8);
        reg |= 3 << 8;
        __raw_writel(reg, S5P_OTHERS);

        reg = __raw_readl(S5P_CLK_OUT);
        reg &= ~(0x1f << 12);
        reg |= 19 << 12;
        __raw_writel(reg, S5P_CLK_OUT);

        reg = __raw_readl(S5P_CLK_OUT);
        reg &= ~0x1;
        reg |= 0x1;
        __raw_writel(reg, S5P_CLK_OUT);
	reg= __raw_readl(S5PV210_ETC2_BASE);
	printk("CRZ ETC2_----------------------------BASE reg=0x%x \n",reg);

	reg= __raw_readl(S5PV210_ETC2_BASE+0x8);
       printk("CRZ ETC2_----------------------------BASE+8 reg=0x%x \n",reg);
	reg=0x5555;
        __raw_writel(reg, (S5PV210_ETC2_BASE+0x8));
	reg= __raw_readl(S5PV210_ETC2_BASE+0x8);
       printk("CRZ ETC2_After----------------------------BASE+8 reg=0x%x \n",reg);

    
#endif
}

#ifdef CONFIG_SENSORS_L3G4200D
#define L3G4200D_DRDY_GPIO S5PV210_GPH0(4)
static int mango210_l3g4200d_init(void)
{
        gpio_request(L3G4200D_DRDY_GPIO, "l3g4200d_irq");
        gpio_direction_input(L3G4200D_DRDY_GPIO);
        return 0;
}

struct l3g4200d_platform_data mango210_gyro_pdata = {
        .poll_interval = 10,
        .gpio_drdy = L3G4200D_DRDY_GPIO,

        .ctrl_reg1 = 0x1f,      /* ODR100 */
        .ctrl_reg2 = 0x00,
        .ctrl_reg3 = 0x08,      /* Enable DRDY interrupt */
        .ctrl_reg4 = 0xA0,      /* BDU enable, 2000 dps */
        .ctrl_reg5 = 0x00,
        .reference = 0x00,
        .fifo_ctrl_reg = 0x00,
        .int1_cfg = 0x00,
        .int1_tsh_xh = 0x00,
        .int1_tsh_xl = 0x00,
        .int1_tsh_yh = 0x00,
        .int1_tsh_yl = 0x00,
        .int1_tsh_zh = 0x00,
        .int1_tsh_zl = 0x00,
        .int1_duration = 0x00,
};
#endif

/* WIFI rfkill gpio platform data */
struct rfkill_gpio_platform_data mango210_wifi_pdata = {
        .reset_gpio     = S5PV210_GPC1(3),
        .shutdown_gpio  = -1,
        .type           = RFKILL_TYPE_WLAN,
        .name           = "mango210-wifi",
};

/* WIFI Platform device */
static struct platform_device mango210_device_wifi = {
        .name           = "rfkill_gpio",
        .id             = -1,
        .dev            = {
                .platform_data  = &mango210_wifi_pdata,
        },
};

static struct platform_device *mango210_devices[] __initdata = {
	&s3c_device_adc,
	&s3c_device_fb,
	&s3c_device_hsmmc0,
	&s3c_device_hsmmc1,
	&s3c_device_hsmmc2,
	&s3c_device_hsmmc3,
	&s3c_device_i2c0,
	&s3c_device_i2c1,
	&s3c_device_i2c2,
#ifdef CONFIG_TOUCHSCREEN_EGALAX
	&s3c_device_i2c5,
#endif
	&s3c_device_rtc,
	&s3c_device_ts,
	&s3c_device_wdt,
	&s5pv210_device_iis0,
	&s5pv210_device_spdif,
	&samsung_asoc_dma,
#ifdef CONFIG_SMSC911X
	&s5p_device_smsc911x,
#endif
	&mango210_buttons_device,
	&mango210_leds_device,
	&mango210_lcd_fb_lcd,
	&s3c_device_timer[3],

#ifdef CONFIG_BATTERY_S3C
        &sec_device_battery,
#endif
	// &mango210_backlight_device,
	&s5p_device_ehci,
	&s5p_device_ohci,
#ifdef CONFIG_USB_GADGET
	&s3c_device_usbgadget,
#endif
#ifdef CONFIG_VIDEO_FIMC
        &s3c_device_fimc0,
        &s3c_device_fimc1,
        &s3c_device_fimc2,
#endif
#ifdef CONFIG_VIDEO_FIMC_MIPI
	&s3c_device_csis,
#endif
#ifdef CONFIG_VIDEO_JPEG_V2
	&s3c_device_jpeg,
#endif
#ifdef CONFIG_VIDEO_MFC50
        &s3c_device_mfc,
#endif
#ifdef CONFIG_ANDROID_PMEM
	&pmem_gpu1_device,
#endif
#ifdef CONFIG_SPI_S3C64XX
	&s5pv210_device_spi0,
	&s5pv210_device_spi1,
#endif

#ifdef CONFIG_REGULATOR
        &mango210_b_pwr_5v,
#endif

#ifdef CONFIG_S5PV210_POWER_DOMAIN
        &s5pv210_pd_tv,
        &s5pv210_pd_lcd,
        &s5pv210_pd_g3d,
        &s5pv210_pd_mfc,
        &s5pv210_pd_audio,
        &s5pv210_pd_cam,  
#endif
	&s3c_device_g3d,
#ifdef CONFIG_VIDEO_G2D
        &s3c_device_g2d,
#endif
#ifdef CONFIG_VIDEO_TV20
        &s5p_device_tvout,
        &s5p_device_cec,
        &s5p_device_hpd,
#endif
	&mango210_device_wifi,
};

/*
 * External camera reset
 * Because the most of cameras take i2c bus signal, so that
 * you have to reset at the boot time for other i2c slave devices.
 * This function also called at fimc_init_camera()
 * Do optimization for cameras on your platform.
*/
#ifdef CAM_ITU_CH_A
static int mango210_cam0_power(int onoff)
{
	int err;
	/* Camera A */
	err = gpio_request(GPIO_PS_VOUT, "GPH0");
	if (err)
		printk(KERN_ERR "failed to request GPH0 for CAM_2V8\n");

	s3c_gpio_setpull(GPIO_PS_VOUT, S3C_GPIO_PULL_NONE);
	gpio_direction_output(GPIO_PS_VOUT, 0);
	gpio_direction_output(GPIO_PS_VOUT, 1);
	gpio_free(GPIO_PS_VOUT);

	return 0;
}
#else
static int mango210_cam1_power(int onoff)
{
	int err;

	/* S/W workaround for the SMDK_CAM4_type board
	 * When SMDK_CAM4 type module is initialized at power reset,
	 * it needs the cam_mclk.
	 *
	 * Now cam_mclk is set as below, need only set the gpio mux.
	 * CAM_SRC1 = 0x0006000, CLK_DIV1 = 0x00070400.
	 * cam_mclk source is SCLKMPLL, and divider value is 8.
	*/

	/* gpio mux select the cam_mclk */
	err = gpio_request(GPIO_PS_ON, "GPJ1");
	if (err)
		printk(KERN_ERR "failed to request GPJ1 for CAM_2V8\n");

	s3c_gpio_setpull(GPIO_PS_ON, S3C_GPIO_PULL_NONE);
	s3c_gpio_cfgpin(GPIO_PS_ON, (0x3<<16));


	/* Camera B */
/*
 * acquire the gpio
 */
	err = gpio_request(GPIO_BUCK_1_EN_A, "GPH0");
	if (err)
		printk(KERN_ERR "failed to request GPH0 for CAM_2V8\n");

	s3c_gpio_setpull(GPIO_BUCK_1_EN_A, S3C_GPIO_PULL_NONE);
/*
 * set the direction of the GPQ as output and write the data
 */
	gpio_direction_output(GPIO_BUCK_1_EN_A, 0);
	gpio_direction_output(GPIO_BUCK_1_EN_A, 1);

	udelay(1000);
/*
 * release the acquired GPQ(1) gpio
 */

	gpio_free(GPIO_PS_ON);
	gpio_free(GPIO_BUCK_1_EN_A);

	return 0;
}
#endif
static int s5k5ba_power_en(int onoff)
{
	if (onoff) {
#ifdef CAM_ITU_CH_A
		mango210_cam0_power(onoff);
#else
		mango210_cam1_power(onoff);
#endif
	} else {
#ifdef CAM_ITU_CH_A
		mango210_cam0_power(onoff);
#else
		mango210_cam1_power(onoff);
#endif
	}

	return 0;
}

#ifdef CONFIG_VIDEO_S5K4EA
/* Set for MIPI-CSI Camera module Power Enable */
static int mango210_mipi_cam_pwr_en(int enabled)
{
	int err;

	err = gpio_request(S5PV210_GPH1(2), "GPH1");
	if (err)
		printk(KERN_ERR "#### failed to request(GPH1)for CAM_2V8\n");

	s3c_gpio_setpull(S5PV210_GPH1(2), S3C_GPIO_PULL_NONE);
	gpio_direction_output(S5PV210_GPH1(2), enabled);
	gpio_free(S5PV210_GPH1(2));

	return 0;
}

/* Set for MIPI-CSI Camera module Reset */
static int mango210_mipi_cam_rstn(int enabled)
{
	int err;

	err = gpio_request(S5PV210_GPH0(3), "GPH0");
	if (err)
		printk(KERN_ERR "#### failed to reset(GPH0) for MIPI CAM\n");

	s3c_gpio_setpull(S5PV210_GPH0(3), S3C_GPIO_PULL_NONE);
	gpio_direction_output(S5PV210_GPH0(3), enabled);
	gpio_free(S5PV210_GPH0(3));

	return 0;
}

/* MIPI-CSI Camera module Power up/down sequence */
static int mango210_mipi_cam_power(int on)
{
	if (on) {
		mango210_mipi_cam_pwr_en(1);
		mdelay(5);
		mango210_mipi_cam_rstn(1);
	} else {
		mango210_mipi_cam_rstn(0);
		mdelay(5);
		mango210_mipi_cam_pwr_en(0);
	}
	return 0;
}
#endif

#ifdef CONFIG_VIDEO_S5K4BA
static struct s5k4ba_platform_data s5k4ba_plat = {
	.default_width = 640,
	.default_height = 480,
	.pixelformat = V4L2_PIX_FMT_UYVY,
	.freq = 44000000,
	.is_mipi = 0,
};

static struct i2c_board_info  s5k4ba_i2c_info = {
	I2C_BOARD_INFO("S5K4BA", 0x2d),
	.platform_data = &s5k4ba_plat,
};

static struct s3c_platform_camera s5k4ba = {
	#ifdef CAM_ITU_CH_A
	.id		= CAMERA_PAR_A,
	#else
	.id		= CAMERA_PAR_B,
	#endif
	.type		= CAM_TYPE_ITU,
	.fmt		= ITU_601_YCBCR422_8BIT,
	.order422	= CAM_ORDER422_8BIT_CBYCRY,
	.i2c_busnum	= 0,
	.info		= &s5k4ba_i2c_info,
	.pixelformat	= V4L2_PIX_FMT_UYVY,
	.srclk_name	= "mout_mpll",
	/* .srclk_name	= "xusbxti", */
	.clk_name	= "sclk_cam1",
	.clk_rate	= 44000000,
	.line_length	= 1920,
	.width		= 640,
	.height		= 480,
	.window		= {
		.left	= 0,
		.top	= 0,
		.width	= 640,
		.height	= 480,
	},

	/* Polarity */
	.inv_pclk	= 0,
	.inv_vsync	= 1,
	.inv_href	= 0,
	.inv_hsync	= 0,

	.initialized	= 0,
	.cam_power	= s5k5ba_power_en,
};
#endif

/* 2 MIPI Cameras */
#ifdef CONFIG_VIDEO_S5K4EA
static struct s5k4ea_platform_data s5k4ea_plat = {
	.default_width = 1920,
	.default_height = 1080,
	.pixelformat = V4L2_PIX_FMT_UYVY,
	.freq = 24000000,
	.is_mipi = 1,
};

static struct i2c_board_info  s5k4ea_i2c_info = {
	I2C_BOARD_INFO("S5K4EA", 0x2d),
	.platform_data = &s5k4ea_plat,
};

static struct s3c_platform_camera s5k4ea = {
	.id		= CAMERA_CSI_C,
	.type		= CAM_TYPE_MIPI,
	.fmt		= MIPI_CSI_YCBCR422_8BIT,
	.order422	= CAM_ORDER422_8BIT_CBYCRY,
	.i2c_busnum	= 0,
	.info		= &s5k4ea_i2c_info,
	.pixelformat	= V4L2_PIX_FMT_UYVY,
	.srclk_name	= "mout_mpll",
	.clk_name	= "sclk_cam0",
	/* .clk_name	= "sclk_cam1", */
	.clk_rate	= 48000000,
	.line_length	= 1920,
	.width		= 1920,
	.height		= 1080,
	.window		= {
		.left	= 0,
		.top	= 0,
		.width	= 1920,
		.height	= 1080,
	},

	.mipi_lanes	= 2,
	.mipi_settle	= 12,
	.mipi_align	= 32,

	/* Polarity */
	.inv_pclk	= 0,
	.inv_vsync	= 1,
	.inv_href	= 0,
	.inv_hsync	= 0,

	.initialized	= 0,
	.cam_power	= mango210_mipi_cam_power,
};
#endif

/* Interface setting */
static struct s3c_platform_fimc fimc_plat_lsi = {
	.srclk_name	= "mout_mpll",
	.clk_name	= "sclk_fimc",
	.lclk_name	= "fimc",
	.clk_rate	= 166750000,
#if defined(CONFIG_VIDEO_S5K4EA)
	.default_cam	= CAMERA_CSI_C,
#else
#ifdef CAM_ITU_CH_A
	.default_cam	= CAMERA_PAR_A,
#else
	.default_cam	= CAMERA_PAR_B,
#endif
#endif
	.camera		= {
#ifdef CONFIG_VIDEO_S5K4ECGX
			&s5k4ecgx,
#endif
#ifdef CONFIG_VIDEO_S5KA3DFX
			&s5ka3dfx,
#endif
#ifdef CONFIG_VIDEO_S5K4BA
			&s5k4ba,
#endif
#ifdef CONFIG_VIDEO_S5K4EA
			&s5k4ea,
#endif
	},
	.hw_ver		= 0x43,
};

#ifdef CONFIG_VIDEO_JPEG_V2
static struct s3c_platform_jpeg jpeg_plat __initdata = {
	.max_main_width	= 800,
	.max_main_height	= 480,
	.max_thumb_width	= 320,
	.max_thumb_height	= 240,
};
#endif

static void __init mango210_smsc9220_init(void)
{
	unsigned int tmp;

	gpio_request(S5PV210_MP01(1), "nCS1");
	s3c_gpio_cfgpin(S5PV210_MP01(1), S3C_GPIO_SFN(2));
	gpio_free(S5PV210_MP01(1));

#define LAN9215_Tacs    (0x0)   // 0clk
#define LAN9215_Tcos    (0x4)   // 4clk
#define LAN9215_Tacc    (0xE)   // 14clk
#define LAN9215_Tcoh    (0x1)   //(0x1) // 1clk
#define LAN9215_Tah     (0x4)   //(0x4) // 4clk
#define LAN9215_Tacp    (0x6)   //(0x6) // tobe zero    // 6clk
#define LAN9215_PMC     (0x0)   // normal(1data)

        tmp = ((LAN9215_Tacs<<28)+(LAN9215_Tcos<<24)+(LAN9215_Tacc<<16)
                +(LAN9215_Tcoh<<12)+(LAN9215_Tah<<8)+(LAN9215_Tacp<<4)+(LAN9215_PMC));

        __raw_writel(tmp, (S5P_SROM_BW + 0x08));

        tmp = __raw_readl(S5P_SROM_BW);
        tmp &= ~(0xf << 4);

        tmp |= (0x1 << 7); // nWBE/nBE(for UB/LB) control for Memory Bank1(0=Not using UB/LB, 1=Using UB/LB)
        tmp |= (0x1 << 6); // Wait enable control for Memory Bank1 (0=WAIT disable, 1=WAIT enable)
        tmp |= (0x1 << 5); // Wait enable control for Memory Bank1 (0=WAIT disable, 1=WAIT enable)
        tmp |= (0x1 << 4); // Data bus width control for Memory Bank1 (0=8-bit, 1=16-bit)
        __raw_writel(tmp, S5P_SROM_BW);

}

static struct i2c_board_info mango210_i2c_devs0[] __initdata = {
	{ I2C_BOARD_INFO("wm8960", 0x1a), },
#ifdef CONFIG_INPUT_BMA150
        {
                I2C_BOARD_INFO("bma150", 0x38),
        },
#endif
#ifdef CONFIG_SENSORS_L3G4200D
        {
        	I2C_BOARD_INFO(L3G4200D_NAME, 0x69),
                .platform_data = &mango210_gyro_pdata,
                .irq = IRQ_EINT(4),
         },

#endif
	#ifdef CONFIG_MANGO_TOUCH_HX8526_7INCH
	/* by crazyboy_edchoi, init mango_cap_touch_ts touch IC */
	{
		I2C_BOARD_INFO("mango_cap_touch_ts", 0x4A),
        .irq            = IRQ_EINT6,
	},
#endif
/* by crazyboy_edchoi, init FT56x6 touch IC */
#if defined(CONFIG_MANGO_TOUCH_FT5x06_7INCH) || defined(CONFIG_MANGO_TOUCH_FT5x06_101INCH)
	{
		I2C_BOARD_INFO("s3c_ts", 0x4A),
        .irq            = IRQ_EINT6,
	},
#endif	

};

static struct i2c_board_info mango210_i2c_devs1[] __initdata = {
#ifdef CONFIG_VIDEO_TV20
        {
                I2C_BOARD_INFO("s5p_ddc", (0x74>>1)),
        },
#endif	
};

static struct i2c_board_info mango210_i2c_devs2[] __initdata = {
#if defined(CONFIG_REGULATOR_MAX8698)
        {
                I2C_BOARD_INFO("max8698", 0xCC >> 1),
                .platform_data  = &mango210_max8698_pdata,
        },
#endif
};

#ifdef CONFIG_SPI_S3C64XX

#define SMDK_MMCSPI_CS 0

static struct s3c64xx_spi_csinfo smdk_spi0_csi[] = {
	[SMDK_MMCSPI_CS] = {
		.line = S5PV210_GPB(1),
		.set_level = gpio_set_value,
		.fb_delay = 0x2,
	},
};

static struct s3c64xx_spi_csinfo smdk_spi1_csi[] = {
	[SMDK_MMCSPI_CS] = {
		.line = S5PV210_GPB(5),
		.set_level = gpio_set_value,
		.fb_delay = 0x2,
	},
};

static struct spi_board_info s3c_spi_devs[] __initdata = {
	{
		.modalias        = "spidev", /* MMC SPI */
		.mode            = SPI_MODE_0,  /* CPOL=0, CPHA=0 */
		.max_speed_hz    = 10000000,
		/* Connected to SPI-0 as 1st Slave */
		.bus_num         = 0,
		.chip_select     = 0,
		.controller_data = &smdk_spi0_csi[SMDK_MMCSPI_CS],
	},
	{
		.modalias        = "spidev", /* MMC SPI */
		.mode            = SPI_MODE_0,  /* CPOL=0, CPHA=0 */
		.max_speed_hz    = 10000000,
		/* Connected to SPI-0 as 1st Slave */
		.bus_num         = 1,
		.chip_select     = 0,
		.controller_data = &smdk_spi1_csi[SMDK_MMCSPI_CS],
	},
};

#endif

static struct s3c2410_ts_mach_info s3c_ts_platform __initdata = {
	.delay			= 10000,
	.presc			= 49,
	.oversampling_shift	= 2,
	.cal_x_max              = 800,
        .cal_y_max              = 480,
        .cal_param              = {
                -13357, -85, 53858048, -95, -8493, 32809514, 65536
        },

};


static void __init mango210_map_io(void)
{
	s5p_init_io(NULL, 0, S5P_VA_CHIPID);
	s3c24xx_init_clocks(24000000);
	s5pv210_gpiolib_init();
	s3c24xx_init_uarts(mango210_uartcfgs, ARRAY_SIZE(mango210_uartcfgs));
#ifndef CONFIG_S5P_HIGH_RES_TIMERS
	s5p_set_timer_source(S5P_PWM2, S5P_PWM4);
#endif
	s5p_reserve_bootmem(s5pv210_media_devs,
                        ARRAY_SIZE(s5pv210_media_devs), S5P_RANGE_MFC);
}

#ifdef CONFIG_S3C_DEV_HSMMC
static struct s3c_sdhci_platdata smdkc110_hsmmc0_pdata __initdata = {
        .max_width              = 4,
        .cd_type                = S3C_SDHCI_CD_INTERNAL,
};      
#endif
 
#ifdef CONFIG_S3C_DEV_HSMMC1
static struct s3c_sdhci_platdata smdkc110_hsmmc1_pdata __initdata = {
        .max_width              = 4,
        .cd_type                = S3C_SDHCI_CD_INTERNAL,
};      
#endif

#ifdef CONFIG_S3C_DEV_HSMMC2
static struct s3c_sdhci_platdata smdkc110_hsmmc2_pdata __initdata = {
        .max_width              = 4,
        .cd_type                = S3C_SDHCI_CD_INTERNAL,
};      
#endif

#ifdef CONFIG_S3C_DEV_HSMMC3
static struct s3c_sdhci_platdata smdkc110_hsmmc3_pdata __initdata = {
        .max_width              = 4,
        .cd_type                = S3C_SDHCI_CD_INTERNAL,
};      
#endif

static void __init smdkc110_setup_clocks(void)
{
        struct clk *pclk;
        struct clk *clk;

        /* set MMC0 clock */
        clk = clk_get(&s3c_device_hsmmc0.dev, "sclk_mmc");
        pclk = clk_get(NULL, "mout_mpll");
        clk_set_parent(clk, pclk);
        clk_set_rate(clk, 50*MHZ);

        pr_info("%s: %s: source is %s, rate is %ld\n",
                                __func__, clk->name, clk->parent->name,
                                clk_get_rate(clk));
}
/* USB EHCI */
static struct s5p_ehci_platdata mango210_ehci_pdata;
static void __init mango210_ehci_init(void)
{
        struct s5p_ehci_platdata *pdata = &mango210_ehci_pdata;

        s5p_ehci_set_platdata(pdata);
}

/* by crazyboy_edchoi, init HX8520touch IC */
#ifdef CONFIG_MANGO_TOUCH_HX8526_7INCH
static void __init hx8526_touch_init(void)
{
    int err;

	err = gpio_request(S5PV210_GPH0(5), "GPH5");
	if (err)
		printk(KERN_ERR "failed to request GPH5 for HX8520 touch IC\n");

	s3c_gpio_setpull(S5PV210_GPH0(5), S3C_GPIO_PULL_NONE);
	gpio_direction_output(S5PV210_GPH0(5), 1);
    mdelay(6);
	gpio_direction_output(S5PV210_GPH0(5), 0);
    mdelay(1);
	gpio_direction_output(S5PV210_GPH0(5), 1);
    mdelay(10);
	gpio_free(S5PV210_GPH0(5));
}
#endif
/* by crazyboy_edchoi, init FT56x6 touch IC */
#if defined(CONFIG_MANGO_TOUCH_FT5x06_7INCH) || defined(CONFIG_MANGO_TOUCH_FT5x06_101INCH)
static void __init ft5x06_touch_init(void)
{
    int err;
	printk("CRZ %s() 10.1 inch touch\n",__func__);
#ifdef CONFIG_MANGO_TOUCH_FT5x06_7INCH
	/* WAKE: GPB6 */
	err = gpio_request(S5PV210_GPB(6), "GPB6");
	if (err)
		printk(KERN_ERR "failed to request GPB6 for FT5x06 touch IC\n");

	s3c_gpio_setpull(S5PV210_GPB(6), S3C_GPIO_PULL_NONE);
	gpio_direction_output(S5PV210_GPB(6), 1);
	gpio_free(S5PV210_GPB(6));
#endif
	err = gpio_request(S5PV210_GPH0(5), "GPH5");
	if (err)
		printk(KERN_ERR "failed to request GPH5 for FT5x06 touch IC\n");

	s3c_gpio_setpull(S5PV210_GPH0(5), S3C_GPIO_PULL_NONE);
	gpio_direction_output(S5PV210_GPH0(5), 1);
    mdelay(5);
	gpio_direction_output(S5PV210_GPH0(5), 0);
    mdelay(2);
	gpio_direction_output(S5PV210_GPH0(5), 1);
	gpio_free(S5PV210_GPH0(5));
}
#endif
/*USB OHCI*/
static struct s5p_ohci_platdata mango210_ohci_pdata;
static void __init mango210_ohci_init(void)
{
        struct s5p_ohci_platdata *pdata = &mango210_ohci_pdata;

        s5p_ohci_set_platdata(pdata);
}

/*USB DEVICE*/
static struct s5p_otg_platdata mango210_otg_pdata;
static void __init mango210_otg_init(void)
{
        struct s5p_otg_platdata *pdata = &mango210_otg_pdata;

        s5p_otg_set_platdata(pdata);
}

#define BACKLIGHT_GPIO  S5PV210_GPD0(0)
#define LCD_EN_GPIO     S5PV210_GPC1(4)

static int lg_fb_lcd_lcd_on(void)
{
    int err;

    /* OnOff LCD_5.0V */
    err = gpio_request(BACKLIGHT_GPIO, "BACKLIGHT");
    err = gpio_request(LCD_EN_GPIO, "LCD_EN");

    if (err) {
        printk(KERN_ERR "failed to request GPIO:%d for "
                        "lcd backlight control\n", BACKLIGHT_GPIO);
        return err;
    }

    gpio_direction_output(BACKLIGHT_GPIO, 1);
    gpio_direction_output(LCD_EN_GPIO, 1);
    s3c_gpio_setpull(LCD_EN_GPIO, S3C_GPIO_PULL_UP);
    gpio_free(BACKLIGHT_GPIO);
    gpio_free(LCD_EN_GPIO);

    return 0;
}

#define MANGO210_WIFI_ENABLE S5PV210_GPC1(1)
#define MANGO210_WIFI_RESET  S5PV210_GPC1(3)

#define MANGO210_BT_ENABLE S5PV210_GPC1(0)
#define MANGO210_BT_RESET  S5PV210_GPC1(2)

static u32 __init mango210_wifi_bt_init(void)
{
    u32 err;

    err = gpio_request(MANGO210_WIFI_ENABLE, "WIFI_ENABLE");
    if (err) {
        printk(KERN_ERR "failed to request GPIO:%d for "
                        "wifi enable\n", MANGO210_WIFI_ENABLE);
        return err;
    }
#if 0
    err = gpio_request(MANGO210_WIFI_RESET, "WIFI_RESET");
    if (err) {
        printk(KERN_ERR "failed to request GPIO:%d for "
                        "wifi reset\n", MANGO210_WIFI_RESET);
        return err;
    }
#endif

    gpio_direction_output(MANGO210_WIFI_ENABLE, 1);
#if 0
    gpio_direction_output(MANGO210_WIFI_RESET, 0);
    mdelay(100);
    gpio_direction_output(MANGO210_WIFI_RESET, 1);
    mdelay(100);
#endif
    gpio_free(MANGO210_WIFI_ENABLE);
#if 0
    gpio_free(MANGO210_WIFI_RESET);
#endif
    return 0;

}


static void __init mango210_machine_init(void)
{
	s3c_pm_init();

	mango210_board_cfg_gpio();

	mango210_wifi_bt_init();

	lg_fb_lcd_lcd_on();

#ifdef CONFIG_SENSORS_L3G4200D
	mango210_l3g4200d_init();
#endif
	mango210_smsc9220_init();
	platform_add_devices(mango210_devices, ARRAY_SIZE(mango210_devices));

#ifdef CONFIG_ANDROID_PMEM
        android_pmem_set_platdata();
#endif

	mango210_sound_init();

	s3c24xx_ts_set_platdata(&s3c_ts_platform);

	s3c_i2c0_set_platdata(NULL);
	s3c_i2c1_set_platdata(NULL);
	s3c_i2c2_set_platdata(NULL);
	i2c_register_board_info(0, mango210_i2c_devs0,
			ARRAY_SIZE(mango210_i2c_devs0));
	i2c_register_board_info(1, mango210_i2c_devs1,
			ARRAY_SIZE(mango210_i2c_devs1));
	i2c_register_board_info(2, mango210_i2c_devs2,
		 	ARRAY_SIZE(mango210_i2c_devs2));
#ifdef CONFIG_TOUCHSCREEN_EGALAX
	i2c_register_board_info(5, i2c_devs5, ARRAY_SIZE(i2c_devs5));
#endif

	s3c_fb_set_platdata(&fb_lcd_fb_data);

#ifdef CONFIG_MANGO_TOUCH_HX8526_7INCH	/* by crazyboy_edchoi, init HX8526 touch IC */
	hx8526_touch_init();
#endif
#ifdef CONFIG_MANGO_TOUCH_FT5x06_101INCH	/* by crazyboy_edchoi, init FT56x6 touch IC */
	ft5x06_touch_init();
#endif
#ifdef CONFIG_S3C_DEV_HSMMC
        s3c_sdhci0_set_platdata(&smdkc110_hsmmc0_pdata);
#endif

#ifdef CONFIG_S3C_DEV_HSMMC1
        s3c_sdhci1_set_platdata(&smdkc110_hsmmc1_pdata);
#endif
#ifdef CONFIG_S3C_DEV_HSMMC2
        s3c_sdhci2_set_platdata(&smdkc110_hsmmc2_pdata);
#endif
#ifdef CONFIG_S3C_DEV_HSMMC3
        s3c_sdhci3_set_platdata(&smdkc110_hsmmc3_pdata);
#endif

#ifdef CONFIG_VIDEO_FIMC
        /* fimc */
        s3c_fimc0_set_platdata(&fimc_plat_lsi);
        s3c_fimc1_set_platdata(&fimc_plat_lsi);
        s3c_fimc2_set_platdata(&fimc_plat_lsi);
#ifdef CAM_ITU_CH_A
	mango210_cam0_power(1);
#else
	mango210_cam1_power(1);
#endif
#endif
#ifdef CONFIG_VIDEO_FIMC_MIPI
	s3c_csis_set_platdata(NULL);
#endif

#ifdef CONFIG_VIDEO_JPEG_V2
	s3c_jpeg_set_platdata(&jpeg_plat);
#endif
#ifdef CONFIG_VIDEO_MFC50
        /* mfc */
        s3c_mfc_set_platdata(NULL);
#endif
        /* spi */
#ifdef CONFIG_SPI_S3C64XX
	if (!gpio_request(S5PV210_GPB(1), "SPI_CS0")) {
	    gpio_direction_output(S5PV210_GPB(1), 1);
	    s3c_gpio_cfgpin(S5PV210_GPB(1), S3C_GPIO_SFN(1));
	    s3c_gpio_setpull(S5PV210_GPB(1), S3C_GPIO_PULL_UP);
	    s5pv210_spi_set_info(0, S5PV210_SPI_SRCCLK_PCLK,
		    ARRAY_SIZE(smdk_spi0_csi));
	}
	if (!gpio_request(S5PV210_GPB(5), "SPI_CS1")) {
	    gpio_direction_output(S5PV210_GPB(5), 1);
	    s3c_gpio_cfgpin(S5PV210_GPB(5), S3C_GPIO_SFN(1));
	    s3c_gpio_setpull(S5PV210_GPB(5), S3C_GPIO_PULL_UP);
	    s5pv210_spi_set_info(1, S5PV210_SPI_SRCCLK_PCLK,
		    ARRAY_SIZE(smdk_spi1_csi));
	}
	spi_register_board_info(s3c_spi_devs, ARRAY_SIZE(s3c_spi_devs));
#endif
#ifdef CONFIG_USB_SUPPORT
        mango210_usb_host_set();
#endif


	mango210_otg_init();
        mango210_ehci_init();
        mango210_ohci_init();
        clk_xusbxti.rate = 24000000;

	 smdkc110_setup_clocks(); 

}

static void __init mango210_fixup(struct machine_desc *mdesc, struct tag *tags, char **cmdline,
                struct meminfo *mi)
{
        mi->bank[0].start = 0x20000000;
        mi->bank[0].size = 256 * SZ_1M;

        mi->bank[1].start = 0x40000000;
        mi->bank[1].size = 256 * SZ_1M;
        /* 1M for ram_console buffer */
        mi->nr_banks = 2;

}

MACHINE_START(SMDKV210, "SMDKV210")
	/* Maintainer: Kukjin Kim <kgene.kim@samsung.com> */
	.boot_params	= S5P_PA_SDRAM + 0x100,
	.init_irq	= s5pv210_init_irq,
        .fixup          = mango210_fixup,
	.map_io		= mango210_map_io,
	.init_machine	= mango210_machine_init,
#ifdef CONFIG_S5P_HIGH_RES_TIMERS
        .timer          = &s5p_systimer,
#else
	.timer		= &s5p_timer,
#endif
MACHINE_END
